==> NJUlogin-2.3.5.tar.gz: NJUlogin-2.3.5/NJUlogin/pwdLogin.py <==
import requests
from lxml import etree
import random
import re
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad
from base64 import b64encode
from inputimeout import inputimeout, TimeoutOccurred
from user_agents import parse
import ddddocr

from .utils import config, urls
from ._base_ import baseLogin


class pwdLogin(baseLogin):
    def __init__(self, username: str, password: str, headers: dict = config.headers, getTimeout: int = config.getTimeout, mobileLogin: bool = False):
        """
        pwdLogin(username: str, password: str, headers: dict = config.headers, getTimeout: int = config.getTimeout)
        @description:
        账号密码登录
        -------
        @param:
        username: str, 账号
        password: str, 密码
        headers: dict, 请求头
        getTimeout: int, 请求超时时间，即在getTimeout秒内未获取到响应则抛出TimeoutError
        mobileLogin: bool, 是否是APP登录
        -------
        """
        if mobileLogin and not parse(headers['User-Agent']).is_mobile:
            print('WARNING: mobileLogin要求使用手机User-Agent，当然你也可以不这样做，但有一定的风险')
        super().__init__(headers)
        self.username = username
        self.password = password
        self.mobileLogin = mobileLogin
        self.getTimeout = getTimeout

    def getCaptcha(self) -> str:
        """获取验证码"""
        captcha = self.get(urls.captcha, timeout=self.getTimeout).content
        ocr = ddddocr.DdddOcr(show_ad=False)
        return ocr.classification(captcha)

    def get_pwdDefaultEncryptSalt(self, selector: etree._Element) -> str:
        """获取密码加密盐"""
        pwdDefaultEncryptSalt = '\n'.join(selector.xpath('//script/text()'))
        return re.search(r'pwdDefaultEncryptSalt = "(.*)";', pwdDefaultEncryptSalt).group(1)

    def pwdEncrypt(self, pwdDefaultEncryptSalt: str) -> str:
        """密码加密"""
        char = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678'
        rds1 = ''.join([random.choice(char) for _ in range(64)])
        rds2 = ''.join([random.choice(char) for _ in range(16)])
        data = (rds1 + self.password).encode('utf-8')
        key = pwdDefaultEncryptSalt.encode('utf-8')
        iv = rds2.encode('utf-8')
        pad_pkcs7 = pad(data, AES.block_size, style='pkcs7')
        aes = AES.new(key, AES.MODE_CBC, iv)
        return b64encode(aes.encrypt(pad_pkcs7)).decode('utf-8')

    def login(self, dest: str) -> requests.Session | None:
        captcha = self.getCaptcha()

        url = urls.login % dest
        html = self.get(url, timeout=self.getTimeout).text
        selector = etree.HTML(html)
        password = self.pwdEncrypt(self.get_pwdDefaultEncryptSalt(selector))

        dataIdx = 1 if parse(self.session.headers['User-Agent']).is_pc else 0
        dllt = 'mobileLogin' if self.mobileLogin else selector.xpath('//input[@name="dllt"]/@value')[dataIdx]
        data = {
            'username': self.username,
            'password': password,
            'lt': selector.xpath('//input[@name="lt"]/@value')[dataIdx],
            'captchaResponse': captcha,
            'dllt': dllt,
            'execution': selector.xpath('//input[@name="execution"]/@value')[dataIdx],
            '_eventId': selector.xpath('//input[@name="_eventId"]/@value')[dataIdx],
            'rmShown': selector.xpath('//input[@name="rmShown"]/@value')[dataIdx]
        }
        res = self.post(url, data=data, timeout=self.getTimeout)

        if res.url == url:
            # 登录失败
            # 可能登录失败也可能是验证码错误
            selector = etree.HTML(res.text)
            try:
                errorMsg = selector.xpath('//span[@id="msg"]/text()')[0]
            except IndexError:
                errorMsg = '需要输入验证码'
                # 需要输入动态码
                res = self.post(urls.dynamicCode,
                    data={'username': self.username, 'authCodeTypeName': 'reAuthDynamicCodeType'},
                    timeout=self.getTimeout
                )
                msg = 'fail'
                try:
                    msg = res.json()['returnMessage']
                    res = res.json()['res']
                    if msg.startswith('您已重复发送'):
                        res = 'success'
                    assert res == 'success'
                except:
                    print('登录失败，%s' % msg)
                    return None
                try:
                    dynamicCode = inputimeout(prompt='请输入发送至手机的动态码: ', timeout=120)
                except TimeoutOccurred:
                    print('登录失败，输入动态码超时')
                    return None
                data = {
                    'dynamicCode': dynamicCode,
                    'username': self.username,
                    'execution': selector.xpath('//input[@name="execution"]/@value')[0],
                    '_eventId': selector.xpath('//input[@name="_eventId"]/@value')[0]
                }
                res = self.post(url, data=data, timeout=self.getTimeout)
                if res.url == url:
                    print('登录失败，动态码错误')
                    return None
                else:
                    print('登录成功')
                    return self.session
            if errorMsg == '无效的验证码':
                # 验证码错误
                return self.login(dest)
            elif errorMsg == '您提供的用户名或者密码有误':
                # 账号密码错误
                print('登录失败，账号密码错误')
                return None
            else:
                print('登录失败，%s' % errorMsg)
                return None

        print('登录成功')
        return self.session

==> NJUlogin-3.1.1.tar.gz: NJUlogin-3.1.1/NJUlogin/pwdLogin.py <==
import requests
from lxml import etree
import random
import re
import time
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad
from base64 import b64encode

from .captchaOCR import CaptchaOCR
from .utils import config, urls
from ._base_ import baseLogin


class pwdLogin(baseLogin):
    def __init__(self, username: str, password: str, headers: dict = config.headers, getTimeout: int = config.getTimeout):
        """
        pwdLogin(username: str, password: str, headers: dict = config.headers, getTimeout: int = config.getTimeout)
        @description:
        账号密码登录
        -------
        @param:
        username: str, 账号
        password: str, 密码
        headers: dict, 请求头
        getTimeout: int, 请求超时时间，即在getTimeout秒内未获取到响应则抛出TimeoutError
        -------
        """
        super().__init__(headers)
        self.username = username
        self.password = password
        self.getTimeout = getTimeout

    def getCaptcha(self) -> str:
        """获取验证码"""
        ms = int(time.time() * 1000) % 1000
        captcha = self.get(urls.captcha % ms, timeout=self.getTimeout).content
        ocr = CaptchaOCR()
        return ocr.get_text(captcha)

    def get_pwdDefaultEncryptSalt(self, selector: etree._Element) -> str:
        """获取密码加密盐"""
        pwdDefaultEncryptSalt = '\n'.join(selector.xpath('//script/text()'))
        return re.search(r'pwdDefaultEncryptSalt = "(.*)";', pwdDefaultEncryptSalt).group(1)

    def pwdEncrypt(self, pwdDefaultEncryptSalt: str) -> str:
        """密码加密"""
        char = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678'
        rds1 = ''.join([random.choice(char) for _ in range(64)])
        rds2 = ''.join([random.choice(char) for _ in range(16)])
        data = (rds1 + self.password).encode('utf-8')
        key = pwdDefaultEncryptSalt.encode('utf-8')
        iv = rds2.encode('utf-8')
        pad_pkcs7 = pad(data, AES.block_size, style='pkcs7')
        aes = AES.new(key, AES.MODE_CBC, iv)
        return b64encode(aes.encrypt(pad_pkcs7)).decode('utf-8')

    def login(self, dest: str = None, trytimes: int = 0) -> requests.Session:
        if dest is not None:
            url = urls.login % dest
        else:
            url = urls.login.split("?")[0]
        html = self.get(url, timeout=self.getTimeout).text
        captcha = self.getCaptcha()
        selector = etree.HTML(html)
        password = self.pwdEncrypt(self.get_pwdDefaultEncryptSalt(selector))

        data = {
            'username': self.username,
            'password': password,
            'lt': selector.xpath('//input[@name="lt"]/@value')[0],
            'captchaResponse': captcha,
            'dllt': selector.xpath('//input[@name="dllt"]/@value')[0],
            'execution': selector.xpath('//input[@name="execution"]/@value')[0],
            '_eventId': selector.xpath('//input[@name="_eventId"]/@value')[0],
            'rmShown': selector.xpath('//input[@name="rmShown"]/@value')[0]
        }
        res = self.post(url, data=data, timeout=self.getTimeout)
        if self.judge_not_login(res, url):
            # print('登录失败')
            selector = etree.HTML(res.text)
            try:
                errorMsg = selector.xpath('//span[@id="msg"]/text()')[0]
            except IndexError:
                print('登录失败，未知错误，可能是需要手机验证码，请先尝试手动登录')
                return None
            if errorMsg == '无效的验证码':
                if trytimes >= 5:
                    print('登录失败，验证码识别错误次数过多')
                    return None
                print('登录失败，验证码识别错误，正在重试')
                return self.login(dest, trytimes + 1)
            else:
                print('登录失败，' + errorMsg)
            return None
        print('登录成功')
        return self.session
